@@ -1,6 +1,6 @@
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
-Components.utils.import("resource://app/modules/urlHelper.jsm");
+Components.utils.import("resource://gre/modules/urlHelper.jsm");
 
 const nsISupports            = Components.interfaces.nsISupports;
 
@@ -55,8 +55,8 @@ function openWindow(parent, url, target, features, args, noExternalArgs) {
   }
   
   // Pass an array to avoid the browser "|"-splitting behavior.
-  var argArray = Components.classes["@mozilla.org/supports-array;1"]
-                    .createInstance(Components.interfaces.nsISupportsArray);
+  var argArray = Components.classes["@mozilla.org/array;1"]
+                    .createInstance(Components.interfaces.nsIMutableArray);
 
   // add args to the arguments array
   var stringArgs = null;
@@ -67,17 +67,17 @@ function openWindow(parent, url, target, features, args, noExternalArgs) {
 
   if (stringArgs) {
     // put the URIs into argArray
-    var uriArray = Components.classes["@mozilla.org/supports-array;1"]
-                       .createInstance(Components.interfaces.nsISupportsArray);
+    var uriArray = Components.classes["@mozilla.org/array;1"]
+                       .createInstance(Components.interfaces.nsIMutableArray);
     stringArgs.forEach(function (uri) {
       var sstring = Components.classes["@mozilla.org/supports-string;1"]
                               .createInstance(nsISupportsString);
       sstring.data = uri;
-      uriArray.AppendElement(sstring);
+      uriArray.appendElement(sstring);
     });
-    argArray.AppendElement(uriArray);
+    argArray.appendElement(uriArray);
   } else {
-    argArray.AppendElement(null);
+    argArray.appendElement(null);
   }
 
   return wwatch.openWindow(parent, url, target, features, argArray);
@@ -122,62 +122,71 @@ nsBlueGriffonContentHandler.prototype = {
     if (!cmdLine.length)
       return;
 
+    var urlArray = [];
     var url = null;
-//@line 127 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
-    if (cmdLine.length == 1) {
-      var arg = cmdLine.getArgument(0);
-      if (arg[0] != "-")
-        url = arg;
-    }
-//@line 133 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
+    var ar = null;
 
-    var ar = cmdLine.handleFlagWithParam("file", false);
-    if (ar || url) {
-      cmdLine.preventDefault = true;
-      var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
-                         .getService(nsIWindowMediator);
-      var e = Services.wm.getEnumerator("bluegriffon");
-      var mostRecent = null;
-      if (e && e.hasMoreElements()) {
-        mostRecent = e.getNext();
+    // deal with -file arguments
+    do {
+      var ar = cmdLine.handleFlagWithParam("file", false);
+      if (ar) {
+        cmdLine.preventDefault = true;
+        try {
+          var localFile = UrlUtils.newLocalFile(ar);
+          var ioService =
+            Components.classes["@mozilla.org/network/io-service;1"]
+                      .getService(Components.interfaces.nsIIOService);
+          var fileHandler =
+            ioService.getProtocolHandler("file")
+                    .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
+          url = fileHandler.getURLSpecFromFile(localFile);
+          urlArray.push(url);
+        }
+        catch(e) {}
       }
-      if (mostRecent) {
-        var localFile = UrlUtils.newLocalFile(ar || url);
-        var ioService =
-          Components.classes["@mozilla.org/network/io-service;1"]
-                    .getService(Components.interfaces.nsIIOService);
-        var fileHandler =
-          ioService.getProtocolHandler("file")
-                   .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
-        var url = fileHandler.getURLSpecFromFile(localFile);
-        var navNav = mostRecent.QueryInterface(nsIInterfaceRequestor)
-                           .getInterface(nsIWebNavigation);
-        var rootItem = navNav.QueryInterface(nsIDocShellTreeItem).rootTreeItem;
-        var rootWin = rootItem.QueryInterface(nsIInterfaceRequestor)
-                              .getInterface(nsIDOMWindow);
-        rootWin.OpenFile(url, true);
-        return;
+    }
+    while (ar);
+
+    do {
+      url = cmdLine.handleFlagWithParam("url", false);
+      if (url)
+        urlArray.push(url);
+    }
+    while(url);
+    
+//@line 158 "/home/glazou/trees/bg-src/bluegriffon/components/bgCommandHandler.js"
+    if (cmdLine.length) {
+      for (var i = 0; i < cmdLine.length; i++) {
+        var arg = cmdLine.getArgument(i);
+        if (arg[0] != "-")
+          urlArray.push(arg);
       }
-      else
-        openWindow(null, gBlueGriffonContentHandler.chromeURL, "_blank", "chrome,dialog=no,all", ar || url);
+    }
+//@line 166 "/home/glazou/trees/bg-src/bluegriffon/components/bgCommandHandler.js"
+
+    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
+               .getService(nsIWindowMediator);
+    var e = Services.wm.getEnumerator("bluegriffon");
+    var mostRecent = null;
+    if (e && e.hasMoreElements()) {
+      mostRecent = e.getNext();
     }
 
-    url = cmdLine.handleFlagWithParam("url", false);
-    if (url) {
+    if (urlArray && urlArray.length) {
       cmdLine.preventDefault = true;
       if (mostRecent) {
         var navNav = mostRecent.QueryInterface(nsIInterfaceRequestor)
-                           .getInterface(nsIWebNavigation);
+                          .getInterface(nsIWebNavigation);
         var rootItem = navNav.QueryInterface(nsIDocShellTreeItem).rootTreeItem;
         var rootWin = rootItem.QueryInterface(nsIInterfaceRequestor)
                               .getInterface(nsIDOMWindow);
-        rootWin.OpenFile(url, true);
+        rootWin.OpenFiles(urlArray, true);
         return;
       }
       return openWindow(null, "chrome://bluegriffon/content/xul/bluegriffon.xul",
-                               "_blank",
-                               "chrome,dialog=no,all",
-                               url);
+                              "_blank",
+                              "chrome,dialog=no,all",
+                              urlArray);
     }
   },
 
@@ -213,84 +222,91 @@ nsDefaultCommandLineHandler.prototype = {
 
   /* nsICommandLineHandler */
   handle : function dch_handle(cmdLine) {
+    if (!cmdLine.length)
+      return;
 
+    var urlArray = [];
     var url = null;
-//@line 219 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
-    if (cmdLine.length == 1) {
-      var arg = cmdLine.getArgument(0);
-      if (arg[0] != "-")
-        url = arg;
+    var ar = null;
+
+    // deal with -file arguments
+    do {
+      var ar = cmdLine.handleFlagWithParam("file", false);
+      if (ar) {
+        cmdLine.preventDefault = true;
+        try {
+          var localFile = UrlUtils.newLocalFile(ar);
+          var ioService =
+            Components.classes["@mozilla.org/network/io-service;1"]
+                      .getService(Components.interfaces.nsIIOService);
+          var fileHandler =
+            ioService.getProtocolHandler("file")
+                    .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
+          url = fileHandler.getURLSpecFromFile(localFile);
+          urlArray.push(url);
+        }
+        catch(e) {}
+      }
     }
-//@line 225 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
+    while (ar);
+
+    do {
+      url = cmdLine.handleFlagWithParam("url", false);
+      if (url)
+        urlArray.push(url);
+    }
+    while(url);
+  
+//@line 261 "/home/glazou/trees/bg-src/bluegriffon/components/bgCommandHandler.js"
+    if (cmdLine.length) {
+      for (var i = 0; i < cmdLine.length; i++) {
+        var arg = cmdLine.getArgument(i);
+        if (arg[0] != "-")
+          urlArray.push(arg);
+      }
+    }
+//@line 269 "/home/glazou/trees/bg-src/bluegriffon/components/bgCommandHandler.js"
 
     var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
-                       .getService(nsIWindowMediator);
+              .getService(nsIWindowMediator);
     var e = Services.wm.getEnumerator("bluegriffon");
     var mostRecent = null;
     if (e && e.hasMoreElements()) {
       mostRecent = e.getNext();
     }
 
-    var ar = cmdLine.handleFlagWithParam("file", false);
-    if (ar || url) {
-       cmdLine.preventDefault = true;
+    if (urlArray && urlArray.length) {
+      cmdLine.preventDefault = true;
       if (mostRecent) {
-        var localFile = UrlUtils.newLocalFile(ar || url);
-        var ioService =
-          Components.classes["@mozilla.org/network/io-service;1"]
-                    .getService(Components.interfaces.nsIIOService);
-        var fileHandler =
-          ioService.getProtocolHandler("file")
-                   .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
-        url = fileHandler.getURLSpecFromFile(localFile);
         var navNav = mostRecent.QueryInterface(nsIInterfaceRequestor)
-                           .getInterface(nsIWebNavigation);
+                          .getInterface(nsIWebNavigation);
         var rootItem = navNav.QueryInterface(nsIDocShellTreeItem).rootTreeItem;
         var rootWin = rootItem.QueryInterface(nsIInterfaceRequestor)
                               .getInterface(nsIDOMWindow);
-        rootWin.OpenFile(url, true);
+        rootWin.OpenFiles(urlArray, true);
         return;
       }
+      return openWindow(null, "chrome://bluegriffon/content/xul/bluegriffon.xul",
+                              "_blank",
+                              "chrome,dialog=no,all",
+                              urlArray);
+    }
 
-//@line 258 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
-      return openWindow(null, "chrome://bluegriffon/content/xul/bluegriffon.xul",
-                               "_blank",
-                               "chrome,dialog=no,all",
-                               ar || url);
-//@line 263 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
-    }
-    url = cmdLine.handleFlagWithParam("url", false);
-    if (url) {
-    	cmdLine.preventDefault = true;
-      if (mostRecent) {
-        var navNav = mostRecent.QueryInterface(nsIInterfaceRequestor)
-                           .getInterface(nsIWebNavigation);
-        var rootItem = navNav.QueryInterface(nsIDocShellTreeItem).rootTreeItem;
-        var rootWin = rootItem.QueryInterface(nsIInterfaceRequestor)
-                              .getInterface(nsIDOMWindow);
-        rootWin.OpenFile(url, true);
-        return;
-      }
-      return openWindow(null, "chrome://bluegriffon/content/xul/bluegriffon.xul",
-                               "_blank",
-                               "chrome,dialog=no,all",
-                               url);
-    }
     if (mostRecent) {
       mostRecent.focus();
       return mostRecent;
     }
-//@line 293 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
+//@line 307 "/home/glazou/trees/bg-src/bluegriffon/components/bgCommandHandler.js"
     if (!cmdLine.preventDefault)
       return openWindow(null, "chrome://bluegriffon/content/xul/bluegriffon.xul",
                                "_blank",
                                "chrome,dialog=no,all",
                                "");
-//@line 299 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
+//@line 313 "/home/glazou/trees/bg-src/bluegriffon/components/bgCommandHandler.js"
 
   },
 
-//@line 317 "/home/daniel/trees/bg-ng/bluegriffon/components/bgCommandHandler.js"
+//@line 331 "/home/glazou/trees/bg-src/bluegriffon/components/bgCommandHandler.js"
 
   helpInfo : ""
 };
