@@ -16,6 +16,10 @@ pref("toolbar.customization.usesheet", true); // true for Mac
 pref("toolbar.customization.usesheet", false); // false otherwise
 #endif
 
+/* disable telemetry */
+pref("toolkit.telemetry.enabled", false);
+pref("toolkit.telemetry.archive.enabled", false);
+
 /* main theme */
 pref("bluegriffon.wysiwyg.theme", "black");
 
@@ -64,6 +68,7 @@ pref("bluegriffon.spellCheck.suggestions", 10);
 pref("bluegriffon.display.comments", true);
 pref("bluegriffon.display.php", true);
 pref("bluegriffon.display.pi", true);
+pref("bluegriffon.display.anchors", true);
 
 // document preferences
 pref("bluegriffon.author", "");
@@ -88,6 +93,7 @@ pref("bluegriffon.defaults.extension.text-html", "html");
 pref("editor.use_css", true);
 pref("bluegriffon.css.policy", "manual");
 pref("bluegriffon.css.prefix", "BG_");
+pref("bluegriffon.css.serialization", "shorthands");
 pref("bluegriffon.css.support.blink", true);
 pref("bluegriffon.css.support.gecko", true);
 pref("bluegriffon.css.support.servo", true);
@@ -101,7 +107,6 @@ pref("bluegriffon.encode_entity", "html");
 pref("bluegriffon.zoom.default", "1");
 
 pref("bluegriffon.history.url_maximum", 10);
-pref("bluegriffon.defaults.restorePreviousSession", true);
 
 pref("signon.rememberSignons", true);
 pref("signon.expireMasterPassword", false);
@@ -129,6 +134,12 @@ pref("bluegriffon.structurebar.class.show", true);
 pref("bluegriffon.structurebar.lang.show", false);
 pref("bluegriffon.structurebar.role.show", true);
 
+// UI visibility
+pref("bluegriffon.ui.structurebar.show", true);
+pref("bluegriffon.ui.statusbar.show", true);
+pref("bluegriffon.ui.vertical_toolbar.show", true);
+pref("bluegriffon.ui.horizontal_toolbars.show", true);
+
 // updates
 pref("bluegriffon.updates.check.enabled", true);
 pref("bluegriffon.updates.frequency", "launch");
@@ -148,19 +159,21 @@ pref("bluegriffon.source.wrap", true);
 pref("bluegriffon.source.wrap.maxColumn", 80);
 pref("bluegriffon.source.wrap.exclude-languages", true);
 pref("bluegriffon.source.wrap.language-exclusions", "");
+pref("bluegriffon.source.zoom.default", "1");
 
 pref("bluegriffon.toolbar.enabled", true);
 pref("bluegriffon.toolbar.icons", "medium");
 pref("bluegriffon.tabs.position", "center");
 
 pref("bluegriffon.osx.dock-integration", true);
+pref("bluegriffon.osx.clipboard.rtf.enabled", true);
 pref("extensions.getAddons.cache.enabled", false);
 
 pref("bluegriffon.css.colors.names.enabled", true);
 pref("bluegriffon.css.colors.type", "hex");
 
 // make links absolute when copied
-pref("clipboard.absoluteLinks", true);
+pref("clipboard.absoluteLinks", false);
 
 pref("extensions.venkman.enableChromeFilter", false);
 
@@ -225,3 +238,26 @@ pref("bluegriffon.responsive.default", "min"); // min, max, minmax
 
 // Blocklist preferences
 pref("extensions.blocklist.enabled", false);
+
+// ARIA
+pref("bluegriffon.aria.epub-type", false);
+
+// File URI Origin policy
+pref("security.fileuri.strict_origin_policy", false);
+
+// Developer Tools related preferences
+pref("devtools.debugger.log", false);
+pref("devtools.chrome.enabled", true);
+pref("devtools.selfxss.count", 5);
+
+// paste dropped images as normal relative URLs instead of data URLs
+pref("bluegriffon.drag_n_drop.images.as_url", true);
+
+// kung fu death grips
+pref("bluegriffon.kungfudeathgrip.shortcuts2017", false);
+
+// last message
+pref("bluegriffon.release_notes.last", "");
+
+pref("browser.preferences.defaultPerformanceSettings.enabled", "false");
+pref("layers.acceleration.disabled", "true");
